#!/usr/bin/env python3

import os
import sys
import serial.tools.list_ports
from test_common import (
    DeviceUnderTestAeadUARTP,
    eprint,
    OpenOcd,
    FileMutex,
    run_nist_lws_aead_test
)


def get_serial():
    ports = serial.tools.list_ports.comports()
    devices = [
        p.device
        for p in ports
        if p.serial_number == 'FT2XCRZ1'
    ]
    devices.sort()
    eprint("Serial port for BluePill is %s" % devices[0])
    return devices[0]


class BluePill(DeviceUnderTestAeadUARTP):
    RAM_SIZE = 0x5000

    def __init__(self, build_dir):
        DeviceUnderTestAeadUARTP.__init__(self)

        self.uart_device = get_serial()
        devname = os.path.basename(self.uart_device)
        self.lock = FileMutex('/var/lock/lwc-compare.%s.lock' % devname)
        self.build_dir = build_dir
        self.template_path = os.path.dirname(sys.argv[0])

        self.firmware_path = os.path.join(
            build_dir, 'firmware.elf')
        self.firmware_bin_path = os.path.join(
            build_dir, 'firmware.bin')
        self.ram_pattern_path = os.path.join(
            self.template_path, 'empty_ram.bin')
        self.ram_dump_path = os.path.join(
            build_dir, 'ram_dump.bin')
        self.openocd_cfg_path = os.path.join(
            self.template_path, 'openocd.cfg')

        self.ocd = OpenOcd(self.openocd_cfg_path)

        self.ser = serial.Serial(
            self.uart_device,
            baudrate=115200,
            timeout=5)

    def firmware_size(self):
        return os.stat(self.firmware_bin_path).st_size

    def flash(self):
        ocd_cmd = 'program %s verify reset' % self.firmware_path
        res = self.ocd.send(ocd_cmd)
        eprint(res)
        assert res == ''
        eprint("Firmware flashed.")

        ocd_cmd = 'reset halt'
        res = self.ocd.send(ocd_cmd)
        eprint(res)
        assert res == ''

        ocd_cmd = 'load_image %s 0x20000000' % self.ram_pattern_path
        res = self.ocd.send(ocd_cmd)
        eprint(res)
        assert res == ''
        eprint("RAM flashed.")

        ocd_cmd = 'resume'
        res = self.ocd.send(ocd_cmd)
        eprint(res)
        assert res == ''

        self.ser = serial.Serial(
            self.uart_device,
            baudrate=115200,
            timeout=5)
        self.reset()

    def reset(self, halt=False):
        ocd_cmd = 'reset halt' if halt else 'reset run'
        res = self.ocd.send(ocd_cmd)
        eprint(res)
        assert res == ''
        eprint("Reset!")

    def dump_ram(self):
        res = self.ocd.send(
            'dump_image %s 0x20000000 0x%x' %
            (self.ram_dump_path, BluePill.RAM_SIZE))
        eprint(res)
        assert res == ''

        eprint("RAM dumped.")
        with open(self.ram_dump_path, 'rb') as ram:
            ram = ram.read()
            if len(ram) != BluePill.RAM_SIZE:
                raise Exception(
                    "RAM dump was %d bytes instead of %d" %
                    (len(ram), BluePill.RAM_SIZE))
            return ram


def main(argv):
    if len(argv) != 2:
        print("Usage: test build_dir")
        return 1

    build_dir = argv[1]
    kat_path = os.path.join(build_dir, 'LWC_AEAD_KAT.txt')

    dut = BluePill(build_dir)
    run_nist_lws_aead_test(dut, kat_path, build_dir, 0x0002)
    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))
